<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\AlteEstabilidadeService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class AlteEstabilidadeController extends Crud {

    protected $dataSourceName = 'gpe_altestabvinc';
    protected $dataSourceNameGet = 'gpe_altestabvinc_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AlteEstabilidadeService $alteEstabilidadeService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->alteEstabilidadeService = $alteEstabilidadeService;
        parent::__construct($dataSourceManager);
    }

    public function getAltestabvinc (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setAltestabvinc (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = $row['NRORG'];
                $nrvinculom = $row['NRVINCULOM'];
                $dtiniestabvinc = $row['DTINIESTABVINC'];
                $dtfimestabvinc = $row['DTFIMESTABVINC'];
                if(!array_key_exists('NRMOTIVOESTAB', $row)){
                    $row['NRMOTIVOESTAB'] = null;
                }
                if(!array_key_exists('NRALTESTABVINC', $row)){
                    $row['NRALTESTABVINC'] = null;
                }
                
                $this->alteEstabilidadeService->verificaPeriodoEstabilidadeVinculo($nrorg, $nrvinculom, $dtiniestabvinc, $dtfimestabvinc, $row['NRALTESTABVINC']);
                
                
                if (array_key_exists('NRALTESTABVINC', $row)) {
                    if ($row['NRALTESTABVINC'] === '') {
                        $row['NRALTESTABVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESTABVINC', 1, 12, true);
                    }
                } else {
                    $row['NRALTESTABVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESTABVINC', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteAltestabvinc(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}