<?php

namespace HCM\Geral\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Util\{ DateUtil, FileUploader, FPA, HCMParameters, Repositories };
use HCM\Geral\Services\MigracaoPlanilhaService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\HTTP\Kernel;

class MigracaoPlanilhaController extends Crud {

    protected $dataSourceNameGet = 'getMigracaoPlanilha';
    protected $dataSourceNameGetLog = 'getLogMigracaoPlanilha';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, EntityManager $entityManager, HCMParameters $hcmParameters, Kernel $httpKernel, FPA $fpa, MigracaoPlanilhaService $migracaoPlanilhaService){
        $this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->httpKernel = $httpKernel;
        $this->migracaoPlanilhaService = $migracaoPlanilhaService;
        parent::__construct($dataSourceManager);
    }

    public function getMigracaoPlanilha(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function getLogMigracaoPlanilha(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGetLog);
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function uploadMigracaoPlanilha(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow()->getArrayCopy();
            
            $nrMigracao = $row['NRMIGRACAO']        ?? null;
            $fileName   = $row['DSPATH'][0]['name'] ?? null;
            
            $filePath = $this->migracaoPlanilhaService->uploadMigracaoPlanilha($row);
            $this->migracaoPlanilhaService->setArquivoMigracao($nrMigracao, $fileName, $filePath);
            $response->addDataSet(new DataSet('uploadMigracaoPlanilha', ['path' => $filePath]));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function deleteMigracaoPlanilha(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow()->getArrayCopy();
            
            $nrMigracao = $row['NRMIGRACAO'] ?? null;
            
            $this->migracaoPlanilhaService->deleteMigracaoPlanilha($nrMigracao);
            $response->addDataSet(new DataSet('deleteMigracaoPlanilha', []));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function getMigracaoPlanilhaLink(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow()->getArrayCopy();
            
            $nrMigracao = $row['NRMIGRACAO'] ?? null;
            
            $filePath = $this->migracaoPlanilhaService->getMigracaoPlanilhaLink($nrMigracao);
            $response->addDataSet(new DataSet('getMigracaoPlanilhaLink', ['path' => $filePath]));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function getModeloPlanilhaLink(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $filePath = $this->migracaoPlanilhaService->getModeloPlanilhaLink();
            $response->addDataSet(new DataSet('getModeloPlanilhaLink', ['path' => $filePath]));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function iniciarMigracaoPlanilha(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->migracaoPlanilhaService->iniciarMigracaoPlanilha();
            $response->addDataSet(new DataSet('iniciarMigracaoPlanilha', []));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 