<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\ModeloaprovsoliService;
use HCM\Geral\Services\SolicitacaoService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;

class ModeloaprovsoliController extends Crud {

    protected $dataSourceName          = 'gpe_modeloaprovsoli';
    protected $dataSourceName_Fase     = 'gpe_faseaprovsoli';
    protected $dataSourceName_setFase = 'gpe_faseaprovsoli_set';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ModeloaprovsoliService $modeloaprovsoliService, SolicitacaoService $solicitacaoService){
        $this->parameterBag            = $parameterBag;
        $this->hcmParameters           = $hcmParameters;
        $this->modeloaprovsoliService  = $modeloaprovsoliService;
        $this->solicitacaoService      = $solicitacaoService;
        parent::__construct($dataSourceManager);
    }

    public function getModeloaprovsoli(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getFaseaprovsoli(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_Fase);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if(isset($row['DSNRPERFILACESSO'])){
                $row['DSNRPERFILACESSO'] = explode(",", $row['DSNRPERFILACESSO']);
                foreach ($row['DSNRPERFILACESSO'] as $chave => $value) {
                    $row['DSNRPERFILACESSO'][$chave] = intval($value);
                }   
                $row['NMPERFILACESSO'] = explode(",", $row['NMPERFILACESSO']);
            }    
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }
    
    public function setModeloaprovsoli(DTO\Request\DataSet $request, DTO\Response $response) {
        $row          = $request->getDataSet()->getRows();
        $row          = $row[0];
        $nrorg        = $this->hcmParameters->getNrorgTrabalho();
        try{
            $row['NRORG']             = $nrorg;
            $row['NRMODELOAPROVSOLI'] = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_MODELOAPROVSOLI', 1, 12, true);
            if(!isset($row['IDATIVO'])){
                $row['IDATIVO'] = 'S';
            }
        parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setFaseaprovsoli(DTO\Request\DataSet $request, DTO\Response $response) {
        $row          = $request->getDataSet()->getRows();
        $row          = $row[0];
        $nrorg        = $this->hcmParameters->getNrorgTrabalho();
        try{
            $row['NRORG'] = $nrorg;
            if ($row['__is_new'] === true) {
                $row['NRFASEAPROVSOLI'] = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_FASEAPROVSOLI', 1, 12, true);
            }
            if(!isset($row['IDATIVO'])){
                $row['IDATIVO'] = 'S';
            }
            if (isset($row['DSNRPERFILACESSO'])){
                if (is_array($row['DSNRPERFILACESSO'])){
                    $row['DSNRPERFILACESSO'] = implode(",",$row['DSNRPERFILACESSO']);
                }
            }
            if (isset($row['NMPERFILACESSO'])){
                if (is_array($row['NMPERFILACESSO'])){
                    $row['NMPERFILACESSO'] = implode(",",$row['NMPERFILACESSO']);
                }
            }

            $outraFase = $this->modeloaprovsoliService->validaOutraFaseMesmaOrdem($row);
            if ($outraFase) {
                $response->addNotification(new Notification("Já existe outra fase com a ordem ".$row['NRORDEM'], Notification::TYPE_ERROR));
            } else {
                $this->dataSourceName = $this->dataSourceName_setFase;
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteModeloaprov(DTO\Request\DataSet $request, DTO\Response $response){
        $row          = $request->getDataSet()->getRows();
        $row          = $row[0];
        $mensagens = $this->modeloaprovsoliService->validaExclusaoModelo($row);
        if($mensagens == 'E'){
            $response->addNotification(new Notification("Não foi possível realizar a exclusão do Modelo pois existem Fases vigentes para o mesmo, por favor primeiro exclua as Fases para prosseguir.", Notification::TYPE_ERROR));
        }else{
            try{
                parent::delete($request, $response);
            } catch(\Exception $e){
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }    
        }
    }
    
    public function deleteFasesaprov(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceName_Fase;

            $utilizada = $this->solicitacaoService->validaFaseSendoUtilizada($request->getDataSet()->getRows()[0]);
            if ($utilizada) {
                $response->addNotification(new Notification("Existem solicitações em processo de aprovação utilizando esse modelo", Notification::TYPE_ERROR));
            } else {
                parent::delete($request, $response);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

} 