<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ProcessoAdmJudController extends Crud {

    protected $ProcessoAdmJudService;
    protected $dataSourceName = '';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters,\HCM\Geral\Services\ProcessoAdmJudService $ProcessoAdmJudService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->ProcessoAdmJudService = $ProcessoAdmJudService;
        parent::__construct($dataSourceManager);
    }

    public function getProcessoAdmJud (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("gpe_ProcessoAdmJud_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getAbDecisao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("gpe_abdecisao_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getMateriaProcessoAdmJud (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("gpe_MateriaProcessoAdmJud_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getSuspensaoProcessoAdmJud (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("gpe_SuspensaoProcessoAdmJud_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
     public function setProcessoAdmJud(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_ProcessoAdmJud";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRPROCESSADMJUD', $row)) {
                    if ($row['NRPROCESSADMJUD'] === '') {
                        $row['NRPROCESSADMJUD'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_PROCESSADMJUD', 1, 12, true); 
                    }
                } else {
                    $row['NRPROCESSADMJUD'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_PROCESSADMJUD', 1, 12, true); 
                }
            }
            //$row[''] = isset($row[''])?$row['']:null;
            $row['SGUFVARAPROADJU'] = isset($row['SGESTADO'])?$row['SGESTADO']:null;
            $row['CDMUNIVARPRADJU'] = isset($row['CDMUNICIPIO'])?$row['CDMUNICIPIO']:null;
            $row['DSOBSERVACAO'] = isset($row['DSOBSERVACAO'])?$row['DSOBSERVACAO']:null;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteProcessoAdmJud(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->ProcessoAdmJudService->validadelete($request->getDataSet()->getRows()[0]);
            $this->dataSourceName = "gpe_ProcessoAdmJud";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    public function getItemProcessoAdmJud (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName("gpe_ItemProcessoAdmJud_get");
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
     public function setItemProcessoAdmJud(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "gpe_ItemProcessoAdmJud";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['NRABDECISAO'] = isset($row['NRABDECISAO'])?$row['NRABDECISAO']:null;
                if (array_key_exists('NRITEMPROCADMJUD', $row)) {
                    if ($row['NRITEMPROCADMJUD'] === '') {
                        $row['NRITEMPROCADMJUD'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_ITEMPROCADMJUD', 1, 12, true); 
                    }
                } else {
                    $row['NRITEMPROCADMJUD'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_ITEMPROCADMJUD', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteItemProcessoAdmJud(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "gpe_ItemProcessoAdmJud";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }
    
    

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}