<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2240Repository {

  protected $conection;

  public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
    $this->connection = $entityManager->getConnection();
    $this->entityManager = $entityManager;
    $this->hcmParameters = $hcmParameters;
  }

  public function getItensParaEnvioS2240($dataset) {
    $sql = "
      SELECT MO.NRMOVIMENTACAO as NRCHAVE,
        VH.NRVINCULOM,
        '' as NRTABELA,
        DECODE(ENV.ENVIOS,NULL,'I', 'A') as IDOPERACAO,
        MO.DTINIMOVIMENT as DTMESCOMPETENC
      FROM GPE_MOVIMENTACAO MO    
      JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = MO.NRVINCULOM AND MO.NRORG = VM.NRORG
      JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                        AND VH.NRVINCULOM = VM.NRVINCULOM
                        AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                  WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                    AND H1.NRORG = VH.NRORG
                                                    AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                    AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
      JOIN SST_RELESTRUTRISCO RTR ON RTR.NRORG = VH.NRORG AND RTR.NRESTRUTURAM = VH.NRESTRUTGEREN                                                 
      /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
      LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
                FROM ESO_ENVIOCADASTROS EC
                JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                  AND EV.NMEVENTO = 'S-2240'
                /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                    AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                    AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                    AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                              WHERE H1.NRORG = EVH.NRORG
                                                                AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                WHERE EC.NRORG = :P_NRORG
                  AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                  AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                  AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                  AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = MO.NRMOVIMENTACAO
      WHERE MO.NRORG = :P_NRORG
      AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM   
      AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
      AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
      AND MO.DTINIMOVIMENT BETWEEN :P_DTMESCOMPETENC AND SYSDATE
      AND (MO.NRMOVIMENTACAO = :P_NRMOVIMENTACAO OR :P_NRMOVIMENTACAO IS NULL)
      AND (MO.DTINIMOVIMENT >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
      ORDER BY 5,2 ";

    $params = array(
      "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
      "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
      "P_CDCNPJEMPREGADOR" => isset($dataset['CDCNPJEMPREGADOR']) ? $dataset['CDCNPJEMPREGADOR'] : null,
      "P_NRESTRUTURAM" => isset($dataset['NRESTRUTURAM']) ? $dataset['NRESTRUTURAM'] : null,
      "P_NRTPAMBIENTE" => isset($dataset['NRTPAMBIENTE']) ? $dataset['NRTPAMBIENTE'] : null,
      "P_NRMOVIMENTACAO" => isset($dataset['NRMOVIMENTACAO']) ? $dataset['NRMOVIMENTACAO'] : null,
      "P_DTREFNAOPERIOD" => isset($dataset['DTREFNAOPERIOD']) ? $dataset['DTREFNAOPERIOD'] : null,
      "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
    );

    return $this->connection->fetchAll($sql, $params);
  }
  
  public function getDadosS2240($parametros) {
    try {
        return $this->getDadosPrincipais($parametros['NRENVIO'], $parametros['NRCHAVE'], $parametros['NRESTRUTURAM']);
    } catch(\Exception $e) {
      error_log($e->getMessage());
      throw $e;
    }
  }

  public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam) {
    $sql = "
/*getDadosS2240*/
SELECT MO.NRMOVIMENTACAO CHAVE,
      'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
      DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
      CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
      REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS cpfTrab,
      CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN NULL ELSE NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) END AS MATRICULA,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN CE.CDESOCIAL ELSE '' END CODCATEG,
      TO_CHAR(MO.DTINIMOVIMENT, 'YYYY-MM-DD') AS dtIniCondicao,
      CASE WHEN SUBSTR(PN.NRINSCRICAOPARC, 0,14) = SUBSTR(PAR.NRINSCRICAOPARC,0,14) OR EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTGEREN THEN 1 ELSE 2 END localAmb,
      EH.NMESTRUTURAH AS dscSetor,
      DECODE(DECODE(EM.NRTIPOESTRUTURA, :P_NRTIPOESTRUTGEREN, PAR.CDTIPOINSCRICAO, PN.CDTIPOINSCRICAO),'CNPJ', '1', 'CAEPF', '3', 'CNO', '4', '') infoAmb_tpInsc,
      DECODE(EM.NRTIPOESTRUTURA, :P_NRTIPOESTRUTGEREN, PAR.NRINSCRICAOPARC, PN.NRINSCRICAOPARC) infoAmb_nrInsc,
      MO.DSOBSERVACAO AS dscAtivDes,
      --respReg
      PHAMB.NRINSCRCONSREG AS nrOc,
      PHAMB.SGUFCONSREG AS ufOC,
      REPLACE(REPLACE(PHAMB.NRCPFPESSOA,'.'),'-') cpfResp,
      MO.NRORG, MO.NRESTRUTURAM, MO.NRVINCULOM, VH.NRCARGO, VH.NRFUNCAO
  FROM GPE_MOVIMENTACAO MO    
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOM VM ON VM.NRORG = MO.NRORG
                      AND VM.NRVINCULOM = MO.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
   /*Categoria Trabalhador*/    
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG                           
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRPESSOA = PE.NRPESSOA
                     AND PH.NRORG = PE.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PE.NRORG)
  /*Ambiente - Estrutura Movimentada*/
  JOIN ESTRUTURAM EM ON EM.NRORG = MO.NRORG
                    AND EM.NRESTRUTURAM = MO.NRESTRUTURAM
  LEFT JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = EH.NRORG)                               
  LEFT JOIN PARCNEGOCIO PN ON PN.NRORG = EH.NRORG
                     AND PN.NRPARCNEGOCIO = EH.NRPARCNEGOCIO
  /*Responsavel Ambiente*/ 
  LEFT JOIN PARCNEGOCIO PARAMB ON PARAMB.NRPARCNEGOCIO = EH.NRPARCREGAMB
                              AND PARAMB.NRORG = EH.NRORG
  LEFT JOIN GPE_PESSOA PAMB ON PAMB.NRPARCNEGOCIO = PARAMB.NRPARCNEGOCIO
                           AND PAMB.NRORG = PARAMB.NRORG
  LEFT JOIN GPE_PESSOAH PHAMB ON PHAMB.NRPESSOA = PAMB.NRPESSOA
                             AND PHAMB.NRORG = PAMB.NRORG
                             AND PHAMB.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                          WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                            AND GPE_PESSOAH.NRPESSOA = PHAMB.NRPESSOA
                                                            AND GPE_PESSOAH.NRORG = PHAMB.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE MO.NRORG = :P_NRORG
   AND MO.NRMOVIMENTACAO = :P_NRCHAVE
    ";

    $params = array(
        "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
        "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
        "P_NRTIPOESTRUTGEREN" => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
        "P_NRENVIO" => $nrenvio,
        "P_NRCHAVE" => $nrchave,
        "P_NRESTRUTURAM" => $nrestruturam
    );

    return $this->connection->fetchAssoc($sql, $params);
  }

  public function getDadosS2240agNoc($parametros) {
    try {
        return $this->getDadosAgNoc($parametros['NRORG'], $parametros['NRESTRUTURAM'], $parametros['NRCARGO'], $parametros['NRFUNCAO']);
    } catch(\Exception $e) {
      error_log($e->getMessage());
      throw $e;
    }
  }

  public function getDadosAgNoc($nrorg, $nrestruturam, $nrcargo, $nrfuncao) {
    $sql = "
    SELECT
      2 tpAval,
      AGNOC.CDESOCIAL codAgNoc,
      AGNOC.DSAGENTENOCIVO dscAgNoc,
      DECODE(RC.NRRISCOEPC, NULL, 1, 2) utilizEPC,
      DECODE(RE.NRRISCOEPI, NULL, 1, 2) utilizEPI
    FROM SST_RELESTRUTRISCO RER
    JOIN SST_FATORRISCO FR ON FR.NRFATORRISCO = RER.NRFATORRISCO
                        AND FR.NRORG = RER.NRORG
    LEFT JOIN SST_RISCOEPI RE ON RE.NRFATORRISCO = FR.NRFATORRISCO
                          AND RE.NRORG = FR.NRORG 
    LEFT JOIN SST_RISCOEPC RC ON RC.NRFATORRISCO = FR.NRFATORRISCO
                          AND RC.NRORG = FR.NRORG
    JOIN ESO_AGENTENOCIVO AGNOC ON AGNOC.NRAGENTENOCIVO = FR.NRAGENTENOCIVO
                               AND (AGNOC.NRORG = FR.NRORG OR AGNOC.NRORG = 0)
    WHERE RER.NRORG = :P_NRORG
    AND RER.NRESTRUTURAM = :P_NRESTRUTURAM
    ";
    $params = array(
        "P_NRORG" => $nrorg,
        "P_NRESTRUTURAM" => $nrestruturam,
        "P_NRCARGO" => $nrcargo,
        "P_NRFUNCAO" => $nrfuncao
    );
    return $this->connection->fetchAll($sql, $params);
  }

  public function getDadosS2240respReg($parametros) {
    try {
        return $this->getDadosRespReg($parametros['NRORG'], $parametros['NRESTRUTURAM'],  $parametros['CHAVE'], $parametros['NRENVIO']);
    } catch(\Exception $e) {
      error_log($e->getMessage());
      throw $e;
    }
  }
  public function getDadosRespReg($nrorg, $nrestruturam, $nrchave, $nrenvio) {
    $sql = "
      SELECT  DECODE(PHAMB.NRCONSELHOREG, 3, 1, 4, 4, 9) AS ideOC,
        PHAMB.NRINSCRCONSREG AS nrOc,
        PHAMB.SGUFCONSREG AS ufOC,
        REPLACE(REPLACE(PHAMB.NRCPFPESSOA,'.'),'-') cpfResp
      FROM GPE_MOVIMENTACAO MO    
      /*Dados do Empregado - Estrutura Movimentada*/
      JOIN ESTRUTURAH EH ON EH.NRORG = MO.NRORG
                        AND EH.NRESTRUTURAM = MO.NRESTRUTURAM
                        AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                  WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                    AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                    AND ESTRUTURAH.NRORG = EH.NRORG)
      /*Responsavel Ambiente*/ 
      LEFT JOIN PARCNEGOCIO PARAMB ON PARAMB.NRPARCNEGOCIO = EH.NRPARCREGAMB
                                  AND PARAMB.NRORG = EH.NRORG
      LEFT JOIN GPE_PESSOA PAMB ON PAMB.NRPARCNEGOCIO = PARAMB.NRPARCNEGOCIO
                              AND PAMB.NRORG = EH.NRORG
      LEFT JOIN GPE_PESSOAH PHAMB ON PHAMB.NRPESSOA = PAMB.NRPESSOA
                                AND PHAMB.NRORG = PAMB.NRORG
                                AND PHAMB.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                              WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                AND GPE_PESSOAH.NRPESSOA = PHAMB.NRPESSOA
                                                                AND GPE_PESSOAH.NRORG = PHAMB.NRORG)
    WHERE MO.NRORG = :P_NRORG
      AND MO.NRMOVIMENTACAO = :P_NRCHAVE
    ";
    $params = array(
        "P_NRORG" => $nrorg,
        "P_NRESTRUTURAM" => $nrestruturam,
        "P_NRENVIO" => $nrenvio,
        "P_NRCHAVE" => $nrchave,
        "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia()
    );
    return $this->connection->fetchAll($sql, $params);
  }
}