<?php

namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\QueryBuilder;
use Teknisa\Libs\Util\Environment;
use Folha\Entities\GpeVinculom;
use HCM\Util\HCM;
use HCM\Util\HCMParameters;

class PeriodoApuracaoService {

    const NRORG_PADRAO = '0';

    protected $entityManager;
    /** @var Environment */
    protected $environment;

    public function __construct(EntityManager $entityManager, Environment $environment, HCMParameters $hcmParameters, HCM $hcm) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->hcmParameters = $hcmParameters;
        $this->hcm = $hcm;
    }

    public function getPeriodoApuracao() {

        $nrvinculom = $this->environment->getNrvinculom();
        $nrorg      = $this->hcmParameters->getNrorgTrabalho();

        /** @var $gpeVinculom \Folha\Entities\GpeVinculom */
        $gpeVinculom = $this->entityManager->getRepository('\Folha\Entities\GpeVinculom')
            ->findOneBy(array('nrvinculom' => $nrvinculom,
                              'nrorg' => $nrorg));

        $nrtipovinculom = $gpeVinculom->getNrtipovinculom();

        $query = $this->entityManager->createQuery("
            SELECT PPA.NRPERIODOAPURACAO,
                   TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY')||' - '||TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY')||
                   ' ('||PPA.IDPERIODOAPURACAO||')'  AS DSPERIODOAPURACAO,
                   PPA.NRORG, PPA.DTFINALAPURACAO, PPA.DTINICIALAPURACAO, PPA.NRTIPOPERIODOAPURACAO, PPA.IDPERIODOAPURACAO
              FROM PTO_PERIODOAPURACAO PPA, PTO_TIPOPERIODOAPURACAO TPA
             WHERE PPA.NRORG                 = $nrorg
               AND TPA.NRTIPOVINCULOM        = $nrtipovinculom
               AND PPA.DTINICIALAPURACAO    <= LAST_DAY(SYSDATE)
               AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
               AND PPA.NRORG                 = TPA.NRORG
             ORDER BY PPA.DTINICIALAPURACAO DESC
        ")->getDQL();

        $periodosApuracao = $this->entityManager->getConnection()->fetchAll($query);

        $periodoApuracaoPonto = array();
        foreach ($periodosApuracao as $periodoApuracao) {
            $periodoApuracaoPonto[] = $this->factory($periodoApuracao);
        }

        return $periodoApuracaoPonto;

    }

    public function factory($periodoApuracao) {

        return array(
            "value" => $periodoApuracao['NRPERIODOAPURACAO'],
            "name"  => $periodoApuracao['DSPERIODOAPURACAO']
        );

    }

    public function getPeriodoApuracaoAtual($parameters, $filters) {
        //$filterPlaceHolder = $this->hcm->getFilterPlaceHolder($filters);
        $nrorg = $parameters['NRORG'];
        $condition = '';
        if(array_key_exists('NRVINCULOM', $parameters)){
            $nrvinculom = str_replace('%', '', $parameters['NRVINCULOM']);
    
            /** @var $gpeVinculom \Folha\Entities\GpeVinculom */
            $gpeVinculom = $this->entityManager->getRepository('\Folha\Entities\GpeVinculom')
                ->findOneBy(array('nrvinculom' => $nrvinculom,
                    'nrorg' => $nrorg));
            
            $nrtipovinculom = $gpeVinculom->getNrtipovinculom();
            
            $condition = "AND TPA.NRTIPOVINCULOM = $nrtipovinculom";
        }

        if (empty($parameters["NRPERIODOAPURACAO"])) {
            $condition .= "";
        } else {
            $condition .= "AND NRPERIODOAPURACAO IN (".implode(',',$parameters["NRPERIODOAPURACAO"]).")";
        }
        $query = $this->entityManager->createQuery("
            SELECT PPA.NRPERIODOAPURACAO,
                   TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY')||' - '||TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY')||
                   ' ('||PPA.IDPERIODOAPURACAO||')'  AS DSPERIODOAPURACAO,
                   PPA.NRORG,
                   TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY')   AS DTFINALAPURACAO,
                   TO_CHAR(PPA.DTINICIALAPURACAO, 'DD/MM/YYYY') AS DTINICIALAPURACAO,
                   PPA.NRTIPOPERIODOAPURACAO,
                   PPA.IDPERIODOAPURACAO
              FROM PTO_PERIODOAPURACAO PPA, PTO_TIPOPERIODOAPURACAO TPA
             WHERE PPA.NRORG = $nrorg
               $condition
               AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
               AND PPA.NRORG = TPA.NRORG
             ORDER BY PPA.DTINICIALAPURACAO DESC
        ")->getDQL();
        $periodoApuracaoAtual = $this->entityManager->getConnection()->fetchAll($query);
        if (isset($periodoApuracaoAtual)) {
            $periodoApuracaoAtual = $this->factoryGetPeriodoApuracao($periodoApuracaoAtual);
            return array($periodoApuracaoAtual);
        } else {
            return false;
        }
    }

    public function factoryGetPeriodoApuracao($periodoApuracao) {

      foreach($periodoApuracao as $index => $periodoApuracao_list){
        $list[$index]= array(
            "NRPERIODOAPURACAO"     => $periodoApuracao_list['NRPERIODOAPURACAO'],
            "DSPERIODOAPURACAO"     => $periodoApuracao_list['DSPERIODOAPURACAO'],
            "DTINICIALAPURACAO"     => $periodoApuracao_list['DTINICIALAPURACAO'],
            "DTFINALAPURACAO"       => $periodoApuracao_list['DTFINALAPURACAO'],
            "NRTIPOPERIODOAPURACAO" => $periodoApuracao_list['NRTIPOPERIODOAPURACAO'],
            "IDPERIODOAPURACAO"     => $periodoApuracao_list['IDPERIODOAPURACAO'],
        );
      }
      
        return $list;
    }

    public function getPeriodoApuracaoAnterior($nrperiodoapuracaocorrente) {
        $nrvinculom = $this->environment->getNrvinculom();
        $nrorg      = $this->environment->getNrOrg();

        $periodoapuracao   = $this->getPeriodoApuracaoAtual(array("NRPERIODOAPURACAO" => $nrperiodoapuracaocorrente));
        $dtinicialapuracao = $periodoapuracao[0]["DTINICIALAPURACAO"];

        /** @var $gpeVinculom \Folha\Entities\GpeVinculom */
        $gpeVinculom = $this->entityManager->getRepository('\Folha\Entities\GpeVinculom')
            ->findOneBy(array('nrvinculom' => $nrvinculom,
                'nrorg' => $nrorg));

        $nrtipovinculom = $gpeVinculom->getNrtipovinculom();

        $query = $this->entityManager->createQuery("
            SELECT PPA.NRPERIODOAPURACAO
              FROM PTO_PERIODOAPURACAO PPA, PTO_TIPOPERIODOAPURACAO TPA
             WHERE PPA.NRORG                 = $nrorg
               AND TPA.NRTIPOVINCULOM        = $nrtipovinculom
               AND PPA.DTFINALAPURACAO       < TO_DATE('$dtinicialapuracao', 'DD/MM/YYYY')
               AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
               AND PPA.NRORG                 = TPA.NRORG
               AND PPA.DTFINALAPURACAO       = (SELECT MAX(DTFINALAPURACAO)
                                                  FROM PTO_PERIODOAPURACAO
                                                 WHERE NRORG = PPA.NRORG
                                                   AND NRTIPOPERIODOAPURACAO = PPA.NRTIPOPERIODOAPURACAO
                                                   AND DTFINALAPURACAO < TO_DATE('$dtinicialapuracao', 'DD/MM/YYYY'))
        ")->getDQL();

        $periodoApuracaoAnterior = $this->entityManager->getConnection()->fetchAll($query);

        if (isset($periodoApuracaoAnterior[0])) {
            return $periodoApuracaoAnterior[0]["NRPERIODOAPURACAO"];
        } else {
            return false;
        }
    }

}