<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\SST\Services\TipoDireitoComiteService;
use HCM\Util\HCMParameters;

class TipoDireitoComiteController extends Crud {

    protected $dataSourceName = 'gpe_tipodireito_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, TipoDireitoComiteService $tipoDireitoComiteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->tipoDireitoComiteService = $tipoDireitoComiteService;
        parent::__construct($dataSourceManager);
    }

    public function getTipoDireito(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setTipoDireito(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRTIPODIREITO', $row)) {
                    if ($row['NRTIPODIREITO'] === '') {
                        $row['NRTIPODIREITO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPODIREITO', 1, 12, true);
                    }
                } else {
                    $row['NRTIPODIREITO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'GPE_TIPODIREITO', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $tipodireitorelac = $this->tipoDireitoComiteService->getTipoDireitoComiteOcup($row['NRTIPODIREITO'], $row['NRORG']);
            if(!isset($tipodireitorelac[0])){
                parent::delete($request, $response);
            }else{
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Esse direito não pode ser apagado! Está vinculado a uma ou mais ocupações na tela "Tipo de Comitê".', DTO\Response\Message::TYPE_WARNING));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 